/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.fieldplan;

import java.io.IOException;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ProjectFactory;
import org.netbeans.spi.project.ProjectState;
import org.openide.filesystems.FileObject;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service=ProjectFactory.class)
public class FieldPlanProjectFactory implements ProjectFactory{
    
    public static final String FIELDPLAN_FILE = "fieldPlan.xml";
    public static final String FOLDER_UNOCCUPIED_PARTFIELDS = "unoccupiedPartFields";
    public static final String FOLDER_PREFIX_PRODUCTION_UNIT = "productionUnit_";

    @Override
    public boolean isProject(FileObject projectDirectory) {
        return projectDirectory.getFileObject(FIELDPLAN_FILE)!= null;
    }

    @Override
    public Project loadProject(FileObject projectDirectory, ProjectState state) throws IOException {
        return isProject(projectDirectory) ? new FieldPlanProject(projectDirectory, state) : null;
    }

    @Override
    public void saveProject(Project project) throws IOException, ClassCastException {
        FileObject projectRoot = project.getProjectDirectory();
        if (projectRoot.getFileObject(FIELDPLAN_FILE) == null) {
            throw new IOException("Project dir " + projectRoot.getPath() +
                    " deleted," +
                    " cannot save project");
        }
        //Force creation of the sensor dir if it was deleted:
        ((FieldPlanProject) project).getFieldPlanFile(true);
    }

}
