/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.fieldplan;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.*;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Timon Veenstra
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-fieldplan")
@NbBundle.Messages({"node_unoccupied_part_fields=Unoccupied fields"})
public class FieldPlanProjectNodeFactory implements NodeFactory {



    @Override
    public NodeList<?> createNodes(Project p) {
        List<Node> nodes = new ArrayList<>();
        // Create node for unoccupied partfields, create folder if not exists
        //
        try {
            FileObject unocFolder = p.getProjectDirectory().getFileObject(FieldPlanProjectFactory.FOLDER_UNOCCUPIED_PARTFIELDS);
            if (unocFolder == null) {
                unocFolder = p.getProjectDirectory().createFolder(FieldPlanProjectFactory.FOLDER_UNOCCUPIED_PARTFIELDS);
            }
            //Get the DataObject that represents it
            DataObject unocObject = DataObject.find(unocFolder);

            //Get its default node-we'll wrap our node around it to change the
            //display name, icon, etc
            Node unocNode = unocObject.getNodeDelegate();

            nodes.add(new UnoccupiedPartFieldsNode(unocNode));
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }
    
    private class UnoccupiedPartFieldsNode extends FilterNode {

        public UnoccupiedPartFieldsNode(Node node) throws DataObjectNotFoundException {
            super(node                                  // the origional node
                    , new FilterNode.Children(node)     // children
                    , new ProxyLookup(node.getLookup()  // lookup from the origional node
                        , Lookups.singleton(new SimpleLayer(node.getDisplayName()))) // lookup with layer
                    );
        }

        @Override
        public Action[] getActions(boolean arg0) {
            Action[] nodeActions = new Action[1];
//            nodeActions[0] = CommonProjectActions.newFileAction();
//            nodeActions[1] = CommonProjectActions.copyProjectAction();
//            nodeActions[2] = CommonProjectActions.deleteProjectAction();
//            nodeActions[5] = CommonProjectActions.setAsMainProjectAction();
//            nodeActions[6] = CommonProjectActions.closeProjectAction();
            return nodeActions;
        }

        /**
         * This implementation only calls clipboardCopy supposing that
         * copy to clipboard and copy by d'n'd are similar.
         *
         * @return transferable to represent this node during a drag
         * @exception IOException when the
         *    cut cannot be performed
         */
        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        /** Copy this node to the clipboard.
         *
         * @return {@link org.openide.util.datatransfer.ExTransferable.Single} with one copy flavor
         * @throws IOException if it could not copy
         * @see NodeTransfer
         */
        @Override
        public Transferable clipboardCopy() throws IOException {
            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage("nl/cloudfarming/client/fieldplan/layers-map-icon.png");
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public String getDisplayName() {
            return NbBundle.getMessage(this.getClass(), "node_unoccupied_part_fields");
        }
    }
}
