/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fieldplan;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

public class PartFieldDataObject extends MultiDataObject implements PropertyChangeListener {

    private InstanceContent ic;
    private Lookup intern;
    private FieldPlanManager fieldPlanManager;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fieldplan");

    public PartFieldDataObject(FileObject fo, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(fo, loader);
        registerEditor(PartField.MIMETYPE, true);

        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        fieldPlanManager = Lookup.getDefault().lookup(FieldPlanManager.class);
        if (null != fieldPlanManager) {
            PartField partField = fieldPlanManager.load(fo);
            assert partField != null;
            LOGGER.log(Level.FINEST, "PartFieldDataObject loaded {0}", System.identityHashCode(partField));
            ic.add(partField);
            ic.add(new SimpleLayer(partField.getName()));
            partField.addPropertyChangeListener(this);
        }
    }

    
    

    @Override
    public void propertyChange(PropertyChangeEvent arg0) {
        SaveCookie saveCookie =
                this.getLookup().
                lookup(SaveCookie.class);
        if (null == saveCookie) {
            this.ic.add(new PartFieldSaveCapability());
        }
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

//    @MultiViewElement.Registration(displayName = "#LBL_PartField_EDITOR",
//    iconBase = "nl/cloudfarming/client/fieldplan/map.png",
//    mimeType = PartField.MIMETYPE,
//    persistenceType = TopComponent.PERSISTENCE_ONLY_OPENED,
//    preferredID = "PartField",
//    position = 1000)
//    @Messages("LBL_PartField_EDITOR=Source")
//    public static MultiViewEditorElement createEditor(Lookup lkp) {
//        return new MultiViewEditorElement(lkp);
//    }
    private class PartFieldSaveCapability implements SaveCookie {

        @Override
        public void save() throws IOException {
            FileObject fo = PartFieldDataObject.this.getPrimaryFile();
            PartField partField = PartFieldDataObject.this.getLookup().
                    lookup(PartField.class);
            PartFieldDataObject.this.fieldPlanManager.save(partField, fo);
            PartFieldDataObject.this.ic.remove(PartFieldSaveCapability.this);
        }
    }
}
