/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.fieldplan;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.SubprojectProvider;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.nodes.Node;

/**
 *
 * @author Timon Veenstra
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-fieldplan")
public class ProductionUnitsNodeFactory implements NodeFactory {

    static String REGISTERED_NODE_LOCATION = "Projects/nl-cloudfarming-client-fieldplan/Nodes";
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.fieldplan");

    @Override
    public NodeList createNodes(Project project) {
        Set<Node> nodes = new HashSet<Node>();

        SubprojectProvider sp = project.getLookup().lookup(SubprojectProvider.class);
        if (sp != null) {
            for (Project p : sp.getSubprojects()) {
                nodes.add(p.getLookup().lookup(LogicalViewProvider.class).createLogicalView());
            }
        }else{
            LOG.log(Level.WARNING, "SubprojectProvider expected in the lookup of Fieldplan project {0}", project.toString());
        }
        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }

}
